#include "parse.h"
#include "net.h"

FILE *log;

void disconnect(int sock)
{
	if(!udp){
		shutdown(sock,SHUT_RDWR);
		close(sock);
		fprintf(stderr,"==[ OK: Disconnected\n");
	}
}

void set_address(char *hname, char *sname, struct sockaddr_in *sap)
{
	struct servent *sp;
	struct hostent *hp;
	char *endptr;
	short port;

	memset(sap,0,sizeof(*sap));
	sap->sin_family = AF_INET;

	if(!inet_aton(hname,&sap->sin_addr)){
		hp = gethostbyname(hname);
		if(hp == NULL){
			fprintf(stderr,"==[ ERR: Unknown host\n");
			return;
		}
		sap->sin_addr = *(struct in_addr *)hp->h_addr;
	}	
	else
		sap->sin_addr.s_addr = inet_addr(hname);
		
	port = strtol(sname,&endptr,0);
	
	if(*endptr==0)
		sap->sin_port = htons(port);
	else{
		if(udp)
			sp = getservbyname(sname,"udp");
		else
			sp = getservbyname(sname,"tcp");
		if(sp == NULL){
			fprintf(stderr,"==[ ERR: Unkown service: %s\n",sname);
			return;
		}
		sap->sin_port = sp->s_port;
	}
}

int connectto(char *addr, char *port)
{
	int sock;

	set_address(addr,port,&a);
	if(!udp){
	
		sock = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);

		if(connect(sock,(struct sockaddr*)&a,sizeof(a))<0){
			fprintf(stderr,"==[ ERR: Couldn't connect with %s:%s\n",
								     addr,port);
			close(sock);
			b0rz_exit(-1,0,0);
		}
		else
			fprintf(stderr,"==[ OK: Connected with %s:%s\n",addr,port);
	}
	else
		sock = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
	return sock;
}

void senddata(int sock, char *data, void *out)
{
	int i,c=0;
	fd_set ff;
	struct timeval tm;
	int r;
	
	FD_ZERO(&ff);
	FD_SET(sock,&ff);
	tm.tv_sec = timeout;
	tm.tv_usec = 0;
	
	if(data){
		if(!udp)
			send(sock,data,strlen(data),0);
		else
			if((r=sendto(sock,data,strlen(data),0,(struct sockaddr*)&a,sizeof(a)))<0)
				b0rz_exit(0,0,0);
		if(out==stdout)
			fprintf(out,"\033[31m%s\033[0m",data);
		else
			fprintf(out,"%s",data);
	}
	if(!udp){
		for(;;){
			if(!(r=select(sock+1,&ff,NULL,NULL,&tm)))
				break;
			if((i=read(sock,&c,1))<=0)
				b0rz_exit(0,1,sock);
			putc(c,out);
		}
	}
	else{
		for(;;){
			if(!(r=select(sock+1,&ff,NULL,NULL,&tm)))
				break;
			if((i=recvfrom(sock,&c,1,0,(struct sockaddr*)&a,(socklen_t*)&a))<=0)
				b0rz_exit(0,1,sock);
			printf("[%d]\n",i);
		}
	}
}

void session(int sock)
{
	FILE *fd;
	char c,buf[LINESZ*391];
	char func[16],arg1[64],arg2[64];
	int i,j,len,sz;
	
	if(!(fd = fopen(tmpf,"r"))){
		fprintf(stderr,"==[ ERR: Couldn't open %s\n",tmpf);
		b0rz_exit(-1,1,sock);
	}
	
	if(lname){
		if(strlen(lname)>16)
			lname[15]=0;
		if(!(log=fopen(lname,"w"))){
			fprintf(stderr,"==[ ERR: Couldn't open %s for "
					  "writing.\n"
			      	 "         Writing to stdout\n",lname);
			bzero(lname,sizeof(lname));
		}
	}
	
	if(lname)
		senddata(sock,NULL,(FILE *)log);
	else
		senddata(sock,NULL,(FILE *)stdout);
	
	len=fstrnum(fd,'\n');
	for(i=0;i<len;i++){
		c=j=0;
		while((c=fgetc(fd))!='\n')
			buf[j++]=c;
		buf[j]='\n';
		if(buf[0]=='@'){ /* these shall be finished in future */
			for(j=0,sz=0;buf[j]!=32 && buf[j]!='\n';j++,sz++)
				func[sz]=buf[j];
			for(j+=1,sz=0;buf[j]!=32 && buf[j]!='\n';j++,sz++)
				arg1[sz]=buf[j];
			for(j+=1,sz=0;buf[j]!=32 && buf[j]!='\n';j++,sz++)
				arg2[sz]=buf[j];
			parse_ext_cmd(func,arg1,arg2);
		}else{
			
			if(getch)getc(stdin);
			
			if(lname)
				senddata(sock,buf,(FILE *)log);
			else
				senddata(sock,buf,(FILE *)stdout);
		}
		memset(buf,0,sizeof(buf));
	}
	fclose(fd);
}
